diff a/src/hotspot/cpu/riscv/smallRegisterMap_riscv.inline.hpp b/src/hotspot/cpu/riscv/smallRegisterMap_riscv.inline.hpp
--- a/src/hotspot/cpu/riscv/smallRegisterMap_riscv.inline.hpp
+++ b/src/hotspot/cpu/riscv/smallRegisterMap_riscv.inline.hpp
@@ -26,27 +26,26 @@
 #define CPU_RISCV_SMALLREGISTERMAP_RISCV_INLINE_HPP
 
 #include "runtime/frame.inline.hpp"
 #include "runtime/registerMap.hpp"
 
+class SmallRegisterMap;
+
 // Java frames don't have callee saved registers (except for fp), so we can use a smaller RegisterMap
-class SmallRegisterMap {
-  constexpr SmallRegisterMap() = default;
-  ~SmallRegisterMap() = default;
-  NONCOPYABLE(SmallRegisterMap);
+template <bool IncludeArgs>
+class SmallRegisterMapType {
+  friend SmallRegisterMap;
+
+  constexpr SmallRegisterMapType() = default;
+  ~SmallRegisterMapType() = default;
+  NONCOPYABLE(SmallRegisterMapType);
 
-public:
-  static const SmallRegisterMap* instance() {
-    static constexpr SmallRegisterMap the_instance{};
-    return &the_instance;
-  }
-private:
   static void assert_is_fp(VMReg r) NOT_DEBUG_RETURN
                                     DEBUG_ONLY({ assert (r == fp->as_VMReg() || r == fp->as_VMReg()->next(), "Reg: %s", r->name()); })
 public:
-  // as_RegisterMap is used when we didn't want to templatize and abstract over RegisterMap type to support SmallRegisterMap
-  // Consider enhancing SmallRegisterMap to support those cases
+  // as_RegisterMap is used when we didn't want to templatize and abstract over RegisterMap type to support SmallRegisterMapType
+  // Consider enhancing SmallRegisterMapType to support those cases
   const RegisterMap* as_RegisterMap() const { return nullptr; }
   RegisterMap* as_RegisterMap() { return nullptr; }
 
   RegisterMap* copy_to_RegisterMap(RegisterMap* map, intptr_t* sp) const {
     map->clear();
@@ -69,11 +68,11 @@
     return nullptr;
   }
 
   bool update_map()    const { return false; }
   bool walk_cont()     const { return false; }
-  bool include_argument_oops() const { return false; }
+  bool include_argument_oops() const { return IncludeArgs; }
   void set_include_argument_oops(bool f)  {}
   bool in_cont()       const { return false; }
   stackChunkHandle stack_chunk() const { return stackChunkHandle(); }
 
 #ifdef ASSERT
